/***************************************************************
/  USB ADVANCE LOADER FOR PS3                                  /
/  Copyright (C) 2009 - Ifcaro <http://ps2dev.ifcaro.net>      /
/                                                              /
/  last mod: 26-04-2009                                        /
/  license:  GNU Library General Public License Version 2      /
/            (see LICENSE on the main directory)               /
***************************************************************/

#include "include/gs.h"
#include "string.h"

extern void *fuente_fnt;
extern int size_fuente_fnt;

gsFontTex* fontTex;

gsFont myFont;

gsDriver myGsDriver; // (320x240 32bit, double-buffer, zbuffer allocated, alpha enabled)
int _ScreenWidth, _ScreenHeight;
extern int PAL;
int zcount=0;

void Graphics(int width, int height){
	
	_ScreenWidth=width;
	_ScreenHeight=height;
	
	if(PAL==1){
		myGsDriver.setDisplayMode(width, height, 170, 80, 
			GS_PSMCT32, 2, GS_TV_PAL, GS_TV_INTERLACE,
			GS_ENABLE, GS_PSMZ32);	
	}else{
		myGsDriver.setDisplayMode(640, 480, 155, 60, 
			GS_PSMCT32, 2, GS_TV_NTSC, GS_TV_INTERLACE,
			GS_ENABLE, GS_PSMZ32);	
	}
		
	myFont.assignPipe(&myGsDriver.drawPipe);
	
	myGsDriver.drawPipe.setAlphaEnable(GS_ENABLE);
}

void flip(){
	zcount=0;

	// Flush the pipe, to draw the prims
	myGsDriver.drawPipe.Flush();

	// Indicate that we have finished drawing
	myGsDriver.DrawBufferComplete();

	// Wait for VSync and then swap buffers
	myGsDriver.WaitForVSync();

	// Display the next frame
	myGsDriver.DisplayNextFrame();

	// Since we're using normal double-buffers, and not swapping on interrupt,
	// then there will always be a next draw buffer available
	myGsDriver.setNextDrawBuffer();
}

void cls(u64 color){
	// Clear the screen (with ZBuffer Disabled)
	myGsDriver.drawPipe.setZTestEnable(GS_DISABLE);
	myGsDriver.drawPipe.RectFlat(0,0,_ScreenWidth,_ScreenHeight,0, color);
	myGsDriver.drawPipe.setZTestEnable(GS_ENABLE);
}

float NTSCFix(float y){
	if(PAL==0){
		return ((y*448)/512);
	}else{
		return y;
	}
}

u64 RGB(int r, int g, int b){
	return GS_SET_RGBA(r,g,b,0x80);
}

u64 RGBA(int r, int g, int b, int a){
	return GS_SET_RGBA(r,g,b,0x80-a);
}

void DrawRect(int x1, int y1, int x2, int y2, u64 color){
	zcount++;
	myGsDriver.drawPipe.RectFlat(x1, (int)NTSCFix(y1), x2, (int)NTSCFix(y2), zcount, color);
}

void DrawRectGoraud(int x1, int y1, int x2, int y2, u64 color1, u64 color2){
	zcount++;
	myGsDriver.drawPipe.RectGouraud(x1, (int)NTSCFix(y1), color1, x1+x2, (int)NTSCFix(y1+y2), color2, zcount);
}

void DrawTriangleGoraud(float x1, float y1, float x2, float y2, float x3, float y3, u64 color1, u64 color2, u64 color3){
	zcount++;
	myGsDriver.drawPipe.TriangleGouraud((int)x1, (int)NTSCFix(y1), zcount, color1, (int)x2, (int)NTSCFix(y2), zcount, color2, (int)x3, (int)NTSCFix(y3), zcount, color3);
}

IIF* LoadImage(char *path){
	IIF* tex_file;
	// Open the file
	int filehandle = fioOpen(path,O_RDONLY);

	// Get the size of the file
	int filesize = fioLseek( filehandle, 0, SEEK_END);

	// allocate space to load the file into EE mem
	tex_file = (IIF*)memalign(64,filesize);

	//seek back to the start of the file
	fioLseek(filehandle, 0, SEEK_SET);

	if (fioRead(filehandle,tex_file,filesize) <= 0)
	{
		printf("Could not load texture.iif\n");
		return NULL;
	}

	fioClose(filehandle);

	printf("Texture = %dx%d\n",tex_file->width,tex_file->height);

	return tex_file;
}

IIF* LoadMemImage(void *buffer, int size){
	IIF* tex_file;

	// allocate space to load the file into EE mem
	tex_file = (IIF*)memalign(64,size);
	
	memcpy(tex_file, buffer, size);

	printf("Texture = %dx%d\n",tex_file->width,tex_file->height);

	return tex_file;
}

void DrawImage(IIF *tex_file, int x, int y, int w, int h){
	DrawImageAlpha(tex_file, x, y, w, h, 0x80);
}

void DrawImageAlpha(IIF *tex_file, int x, int y, int w, int h, int a){
	zcount++;
	// Upload into the beginning of texture mem (with texture-buffer width set to 256)
	myGsDriver.drawPipe.TextureUpload(
		myGsDriver.getTextureBufferBase(), 256,
		0, 0, (int)tex_file->psm, (const unsigned char*)&(tex_file->pixel_data),
		tex_file->width, tex_file->height);

	// Set-up the Texture (can do this outside the draw loop in this example, since we are only using one texture)
	myGsDriver.drawPipe.TextureSet(
		myGsDriver.getTextureBufferBase(), 256,
		GS_TEX_SIZE_256, GS_TEX_SIZE_256, tex_file->psm, 
		0, 0, 0, 0);

	// Draw a moving, solid, textured rectangle, with our texture on it (at z=1)
	myGsDriver.drawPipe.RectTexture(
		x, (int)NTSCFix(y), 0, 0,
		x+w, (int)NTSCFix(y+h), tex_file->width, tex_file->height,
		zcount, GS_SET_RGBA(0x80,0x80,0x80,a));
		
}

void LoadFont(){
	// allocate space to load the file into EE mem
	fontTex = (gsFontTex*)memalign(64,size_fuente_fnt);
	memcpy(fontTex,&fuente_fnt,size_fuente_fnt);

}

void LoadFontFile(){
	// Open the file
	int filehandle = fioOpen("host:gfx/arial.fnt",O_RDONLY);

	// Get the size of the file
	int filesize = fioLseek( filehandle, 0, SEEK_END);

	// allocate space to load the file into EE mem
	fontTex = (gsFontTex*)memalign(64,filesize);

	//seek back to the start of the file
	fioLseek(filehandle, 0, SEEK_SET);

	if (fioRead(filehandle,fontTex,filesize) <= 0)
	{
		printf("Could not load texfont.fnt\n");
		while(1);
	}

	fioClose(filehandle);
}

void DrawText(char *txt, int x, int y, u64 color, int centrado){
	DrawTextBox(txt, x, y, _ScreenWidth-30, color, centrado);
}

void DrawTextBox(char *txt, int x, int y, int w, u64 color, int centrado){
	gsFontAlign align=GSFONT_ALIGN_LEFT;
	
	if(centrado==0){
		align=GSFONT_ALIGN_LEFT;
	}else if(centrado==1){
		align=GSFONT_ALIGN_CENTRE;
	}else if(centrado==2){
		align=GSFONT_ALIGN_RIGHT;
	}

	zcount++;
	// Upload into the beginning of texture mem (with texture-buffer width set to 256)
	myFont.uploadFont(fontTex, myGsDriver.getTextureBufferBase(), 
		fontTex->TexWidth, // Use the fontTex width as texbuffer width (can use diff width)
		0, 0 );
	myFont.Print(x, w, (int)NTSCFix(y), zcount, color, align, txt);
}
